@@ -19,12 +19,13 @@ rm -rf $INSTALL_FOLDER
 # Note: BSD sed -i needs an argument unders OSX
 # so first renaming to .bak and then deleting backup files
 find loky -name "*.py" | xargs sed -i.bak "s/from loky/from joblib.externals.loky/"
-find loky -name "*.bak" | xargs rm
 
 for f in $(git grep -l "cloudpickle" loky); do
     echo $f;
-    sed -i 's/import cloudpickle/from joblib.externals import cloudpickle/' $f
-    sed -i 's/from cloudpickle import/from joblib.externals.cloudpickle import/' $f
+    sed -i.bak 's/import cloudpickle/from joblib.externals import cloudpickle/' $f
+    sed -i.bak 's/from cloudpickle import/from joblib.externals.cloudpickle import/' $f
 done
 
-sed -i "s/loky.backend.popen_loky/joblib.externals.loky.backend.popen_loky/" loky/backend/popen_loky_posix.py
+sed -i.bak "s/loky.backend.popen_loky/joblib.externals.loky.backend.popen_loky/" loky/backend/popen_loky_posix.py
+
+find loky -name "*.bak" | xargs rm
